@@ -1,14 +1,13 @@
-# coding=utf-8
+# -*- coding: utf-8 -*-
 import os
 
-from django.core.management.base import CommandError
+from django.core.management.base import BaseCommand, CommandError
 from django.core.management.color import color_style
 from django.template.loader import get_template
 
 from django_extensions.compat import add_to_builtins_compat
 from django_extensions.management.utils import signalcommand
 from django_extensions.utils import validatingtemplatetags
-from django_extensions.compat import CompatibilityBaseCommand as BaseCommand
 
 
 #
@@ -21,6 +20,7 @@ class Command(BaseCommand):
     help = "Validate templates on syntax and compile errors"
 
     def add_arguments(self, parser):
+        super(Command, self).add_arguments(parser)
         parser.add_argument(
             '--break', '-b', action='store_true', dest='break',
             default=False, help="Break on first error.")
@@ -32,7 +32,7 @@ class Command(BaseCommand):
             '--force-new-urls', '-n', action='store_true',
             dest='force_new_urls',
             default=False, help="Error on usage of old style url tags "
-            "(without {% load urls from future %}")
+            "(without {%% load urls from future %%}")
         parser.add_argument(
             '--include', '-i', action='append', dest='includes',
             default=[], help="Append these paths to TEMPLATE_DIRS")
